import logging
from Products.PluggableAuthService.PluggableAuthService import \
    registerMultiPlugin
from dm.zope.saml2.spsso.plugin import IntegratedSimpleSpssoPlugin
from dm.zope.saml2.permission import manage_saml
from plone import api

logger = logging.getLogger('cnm.saml2')


class CnmSpssoPlugin(IntegratedSimpleSpssoPlugin):
    meta_type = "Saml CNM integrated simple spsso plugin"

    def format_user_id(self, info):
        # return username part of the email address
        user_id = info['nameid'].split('@')[0]
        self.set_user_and_groups(user_id, info)
        logger.info(f"{info['nameid']} logged in")
        return user_id

    def set_user_and_groups(self, user_id, info):
        # create Plone user if they don't exist
        # make sure they are not in any groups
        if api.user.get(username=user_id):
            return
        user = api.user.create(
            username=user_id,
            email=info['nameid'],
            properties=dict()
        )
        logger.info(f'Created user {user_id}')

        # reset roles and groups
        roles = api.user.get_roles(user=user)
        for role in roles:
            if role != 'Authenticated':
                api.user.revoke_roles(user=user, roles=[role])
        groups = api.group.get_groups(user=user)
        for group in groups:
            if group.id != 'AuthenticatedUsers':
                api.group.remove_user(group=group, user=user)


def initialize(context):
    from dm.zope.schema.z2.constructor import add_form_factory, \
        SchemaConfiguredZmiAddForm

    registerMultiPlugin(CnmSpssoPlugin.meta_type)

    context.registerClass(
        CnmSpssoPlugin,
        constructors=(add_form_factory(CnmSpssoPlugin, form_class=SchemaConfiguredZmiAddForm),),
        permission=manage_saml,
        visibility=None,
    )
